# Copyright 2021 CR-Suite Development Team
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from jax import jit, grad, lax

import jax.numpy as jnp
import cr.nimble as cnb

from .smooth import build2

def smooth_linear(C, D=0.):
    r"""Linear function and its gradient :math:`f(x) = \langle c, x \rangle + d`
    """
    C = jnp.asarray(C)
    D = jnp.asarray(D)
    C, D = cnb.promote_arg_dtypes(C, D)

    @jit
    def func(x):
        x = jnp.asarray(x)
        x = cnb.promote_arg_dtypes(x)
        return cnb.arr_rdot(C, x) + D

    @jit
    def gradient(x):
        return C

    return build2(func, gradient)

